"""fhs_iptv_tools - foxhunt software iptv tools, making iptv easier versions """

__version__ = "0.8.3"
__author__ = "Richard de Vos <rdevos72@gmail.com>"
__author_name__ = "Richard de Vos"
__author_email__ = "rdevos72@gmail.com"
__app_name__ = "fhs_iptv_tools"
__project_name__ = "fhs-iptv-tools"
__all__ = []
